﻿Imports System.Math

Public Class Form1
    Private m_DrawingBox As Boolean
    Private m_StartX, m_StartY, m_CurX, m_CurY As Double

    Private m_Xmin, m_Xmax, m_Ymin, m_Ymax As Double

    Public MaxIterations As Integer
    Public C As Complex

    Public NumColors As Integer
    Private m_Colors() As Color

    Private m_Bm As Bitmap

    Private Const MIN_X As Double = -2
    Private Const MAX_X As Double = 2
    Private Const MIN_Y As Double = -2
    Private Const MAX_Y As Double = 2

    ' Return this color's value.
    ReadOnly Property Color(ByVal Index As Integer) As Color
        Get
            Color = m_Colors(Index)
        End Get
    End Property

    ' Reset the number of colors to 0.
    Public Sub ResetColors()
        NumColors = 0
        Erase m_Colors
    End Sub

    ' Add this color to the list.
    Public Sub AddColor(ByVal new_color As Color)
        NumColors += 1
        ReDim Preserve m_Colors(NumColors - 1)
        m_Colors(NumColors - 1) = new_color
    End Sub

    ' Adjust the aspect ratio of the selected
    ' coordinates so they fit the window properly.
    Private Sub AdjustAspect()
        Dim want_aspect As Double = (m_Ymax - m_Ymin) / (m_Xmax - m_Xmin)
        Dim picCanvas_aspect As Double = picCanvas.ClientSize.Height / picCanvas.ClientSize.Width
        Dim wid, mid, hgt As Double
        If want_aspect > picCanvas_aspect Then
            ' The selected area is too tall and thin.
            ' Make it wider.
            wid = (m_Ymax - m_Ymin) / picCanvas_aspect
            mid = (m_Xmin + m_Xmax) / 2
            m_Xmin = mid - wid / 2
            m_Xmax = mid + wid / 2
        Else
            ' The selected area is too short and wide.
            ' Make it taller.
            hgt = (m_Xmax - m_Xmin) * picCanvas_aspect
            mid = (m_Ymin + m_Ymax) / 2
            m_Ymin = mid - hgt / 2
            m_Ymax = mid + hgt / 2
        End If
    End Sub

    ' Draw the Vortex set.
    Private Sub DrawVortex()
        ' Work until the magnitude squared > 4.
        Const MAX_MAG_SQUARED As Integer = 4

        ' Make a Bitmap to draw on.
        m_Bm = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        Using gr As Graphics = Graphics.FromImage(m_Bm)
            ' Clear.
            gr.Clear(picCanvas.BackColor)
            picCanvas.Image = m_Bm
            Application.DoEvents()

            ' Adjust the coordinate bounds to fit picCanvas.
            AdjustAspect()

            ' dReaC is the change in the real part
            ' (X value) for C. dImaC is the change in the
            ' imaginary part (Y value).
            Dim wid As Integer = picCanvas.ClientRectangle.Width
            Dim hgt As Integer = picCanvas.ClientRectangle.Height
            Dim dReaC As Double = (m_Xmax - m_Xmin) / (wid - 1)
            Dim dImaC As Double = (m_Ymax - m_Ymin) / (hgt - 1)

            ' Calculate the values.
            Dim Zrea As Double = m_Xmin
            For X = 0 To wid - 1
                Dim Zima As Double = m_Ymin
                For Y = 0 To hgt - 1
                    Dim ZN As New Complex(Zrea, Zima)
                    Dim ZNminus1 As New Complex(Zrea, Zima)
                    Dim clr As Integer = 1
                    Do While clr < MaxIterations And ZN.MagnitudeSquared < MAX_MAG_SQUARED
                        ' Calculate Z(clr).
                        Dim ZNplus1 As Complex = ZN * ZN + C.Re + C.Im * ZNminus1
                        ZNminus1 = ZN
                        ZN = ZNplus1
                        clr += 1
                    Loop

                    ' Set the pixel's value.
                    m_Bm.SetPixel(X, Y, m_Colors(clr Mod NumColors))

                    Zima += dImaC
                Next Y
                Zrea += dReaC

                ' Let the user know we're not dead.
                If X Mod 10 = 0 Then
                    picCanvas.Refresh()
                End If
            Next X
        End Using

        Text = "Vortex (" & _
            m_Xmin.ToString("0.000000") & ", " & _
            m_Ymin.ToString("0.000000") & ")-(" & _
            m_Xmax.ToString("0.000000") & ", " & _
            m_Ymax.ToString("0.000000") & ")"
    End Sub

    ' Scale the selected area by this factor.
    Private Sub ScaleArea(ByVal scale_factor As Integer)
        Dim size As Double = scale_factor * (m_Xmax - m_Xmin)
        If size > 3.2 Then
            mnuScaleFull_Click(Nothing, Nothing)
            Exit Sub
        End If
        Dim mid As Double = (m_Xmin + m_Xmax) / 2
        m_Xmin = mid - size / 2
        m_Xmax = mid + size / 2

        size = scale_factor * (m_Ymax - m_Ymin)
        If size > 2.4 Then
            mnuScaleFull_Click(Nothing, Nothing)
            Exit Sub
        End If
        mid = (m_Ymin + m_Ymax) / 2
        m_Ymin = mid - size / 2
        m_Ymax = mid + size / 2

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawVortex()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub

    Private Sub mnuScale_2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_2.Click
        ScaleArea(2)
    End Sub
    Private Sub mnuScale_4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_4.Click
        ScaleArea(4)
    End Sub
    Private Sub mnuScale_8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_8.Click
        ScaleArea(8)
    End Sub
    ' Zoom out to full scale.
    Private Sub mnuScaleFull_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScaleFull.Click
        m_Xmin = MIN_X
        m_Xmax = MAX_X
        m_Ymin = MIN_Y
        m_Ymax = MAX_Y

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawVortex()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Show()
        Application.DoEvents()

        MaxIterations = 64
        C = New Complex(0.62, -0.55)

        ' Create some default colors.
        ResetColors()
        Dim frm As New VortexConfig()
        AddColor(frm.picColor_40.BackColor)
        AddColor(frm.picColor_17.BackColor)
        AddColor(frm.picColor_18.BackColor)
        AddColor(frm.picColor_19.BackColor)
        AddColor(frm.picColor_20.BackColor)
        AddColor(frm.picColor_21.BackColor)
        AddColor(frm.picColor_22.BackColor)
        AddColor(frm.picColor_23.BackColor)
        frm.Close()

        ' Display the first Vortex set.
        mnuScaleFull_Click(Nothing, Nothing)
    End Sub

    Private Sub mnuOptOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuOptOptions.Click
        Dim frm As New VortexConfig()
        frm.Initialize(Me)
        frm.ShowDialog()
    End Sub

    ' Refresh. This is useful when the user resizes the form.
    Private Sub mnuScaleRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScaleRefresh.Click
        ScaleArea(1)
    End Sub

    Private Sub picCanvas_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseDown
        m_DrawingBox = True
        m_StartX = e.X
        m_StartY = e.Y
        m_CurX = e.X
        m_CurY = e.Y
    End Sub

    Private Sub picCanvas_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseMove
        If Not m_DrawingBox Then Exit Sub

        m_CurX = e.X
        m_CurY = e.Y

        Dim bm As New Bitmap(m_Bm)
        Using gr As Graphics = Graphics.FromImage(bm)
            gr.DrawRectangle(Pens.Yellow, _
                CInt(Min(m_StartX, m_CurX)), CInt(Min(m_StartY, m_CurY)), _
                CInt(Abs(m_StartX - m_CurX)), CInt(Abs(m_StartY - m_CurY)))
        End Using
        picCanvas.Image = bm
    End Sub

    Private Sub picCanvas_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseUp
        If Not m_DrawingBox Then Exit Sub
        m_DrawingBox = False
        picCanvas.Image = m_Bm

        m_CurX = e.X
        m_CurY = e.Y

        ' Put the coordinates in proper order.
        Dim x1 As Double = Min(m_StartX, m_CurX)
        Dim x2 As Double = Max(m_StartX, m_CurX)
        If x1 = x2 Then x2 = x1 + 1

        Dim y1 As Double = Min(m_StartY, m_CurY)
        Dim y2 As Double = Max(m_StartY, m_CurY)
        If y1 = y2 Then y2 = y1 + 1

        ' Convert screen coords into drawing coords.
        Dim factor As Double = (m_Xmax - m_Xmin) / picCanvas.ClientSize.Width
        m_Xmax = m_Xmin + x2 * factor
        m_Xmin = m_Xmin + x1 * factor

        factor = (m_Ymax - m_Ymin) / picCanvas.ClientSize.Height
        m_Ymax = m_Ymin + y2 * factor
        m_Ymin = m_Ymin + y1 * factor

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawVortex()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub
End Class
